--[[
	Copyright (C) Udorn (Blackhand)
--]]

petbm = petbm or {}

petbm.PetPortrait = {}

local L = petbm.Locale.GetInstance()
local log = petbm.Debug:new("PetPortrait")
local nextId = 0

local function _IsMissing(self, petIndex)
	local quality = C_PetBattles.GetBreedQuality(LE_BATTLE_PET_ENEMY, petIndex)
	local speciesId = C_PetBattles.GetPetSpeciesID(LE_BATTLE_PET_ENEMY, petIndex)
	local rarity = petbm.PetUtil.GetPetRarityFromJournal(speciesId) or -1
--	log:Debug("IsMissing quality [%s] speciesId [%s] journal rarity [%s]", quality, speciesId, rarity)
	return quality > rarity
end

-- Update the pet information according to the petIndex and petOwner. 
-- Returns petType, name, level, health, maxHealth, quality, icon or nil
local function Update(self, petOwner, petIndex)
	
	if (not petOwner or not petIndex) then
		return
	end
	
	if (petIndex > C_PetBattles.GetNumPets(petOwner)) then
		return
	end
	
	local icon = C_PetBattles.GetIcon(petOwner, petIndex)
	local level = C_PetBattles.GetLevel(petOwner, petIndex)
	local petType = C_PetBattles.GetPetType(petOwner, petIndex)
	local quality = C_PetBattles.GetBreedQuality(petOwner, petIndex)
	local health = C_PetBattles.GetHealth(petOwner, petIndex)
	local maxHealth = C_PetBattles.GetMaxHealth(petOwner, petIndex)
	local name = C_PetBattles.GetName(petOwner, petIndex)
		
	self.frame.icon:SetTexture(icon)
	self.frame.level:SetText(level)
	self.frame.petType:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])

	local isWild = C_PetBattles.IsWildBattle()		
	if (petOwner == LE_BATTLE_PET_ALLY or isWild ~= true or not _IsMissing(self, petIndex)) then
		self.frame.catch:Hide()
		self.frame.catchBack:Hide()
	else
		self.frame.catchBack:Show()
		self.frame.catch:Show()
	end
	
	return petType, name, level, health, maxHealth, quality, icon
end

local function _InitFrame(self, parent, size, isLeft)
	local frame = CreateFrame("Button", nil, parent)
	frame:SetWidth(size)
	frame:SetHeight(size)
	frame:EnableMouse(false)
	frame.obj = self
	frame.tab = tab
	self.frame = frame
	
	-- background
	local texture = frame:CreateTexture(nil, "BORDER")
	texture:SetAllPoints(frame)
	texture:SetTexture("Interface\\PetBattles\\PetJournal")
	texture:SetTexCoord(0.41992188, 0.52343750, 0.02246094, 0.07519531)
	
	-- portrait icon
	local icon = frame:CreateTexture(nil, "BACKGROUND")
	icon:SetWidth(size - 18)
	icon:SetHeight(size - 18)
	icon:SetPoint("TOPLEFT", 10, -8)
	if (not isLeft) then
		icon:SetTexCoord(0, 1, 0, 1)
	end
	frame.icon = icon
	
	-- level
	local levelBack = frame:CreateTexture(nil, "OVERLAY", "MainPet-LevelBubble")
	levelBack:SetWidth(20)
	levelBack:SetHeight(20)
	levelBack:ClearAllPoints()
	levelBack:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 3, -3)

	local level = frame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	level:SetJustifyH("CENTER")
	level:SetPoint("CENTER", levelBack, "CENTER", 0, 0)
	level:SetText("99")
	frame.level = level

	-- type
	local typeBack = frame:CreateTexture(nil, "OVERLAY")
	typeBack:SetWidth(20)
	typeBack:SetHeight(20)
	typeBack:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", -3, -3)
	typeBack:SetTexCoord(0.79687500, 0.49218750, 0.50390625, 0.65625000)
	typeBack:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[1])
	frame.petType = typeBack

	-- catch
	local catchBack = frame:CreateTexture(nil, "ARTWORK", "MainPet-LevelBubble")
	catchBack:SetWidth(20)
	catchBack:SetHeight(20)
	catchBack:ClearAllPoints()
	catchBack:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 3, 3)
	frame.catchBack = catchBack
	local catchName = "petbmCatch"..nextId
	nextId = nextId + 1
	local catch = frame:CreateTexture(catchName, "OVERLAY")
	catch:SetWidth(15)
	catch:SetHeight(15)
	catch:SetPoint("CENTER", catchBack, "CENTER", 0, 0)
	SetPortraitToTexture(catchName, "Interface\\Icons\\INV_Pet_PetTrap")
	frame.catch = catch
end

local function New(parent, size, isLeft)
	local instance = {}
		
	instance.Update = Update
	
	_InitFrame(instance, parent, size, isLeft)
	
	return instance
end

petbm.PetPortrait.New = New