--[[
	Copyright (C) Udorn (Blackhand)
--]]

petbm = petbm or {}

petbm.PetUtil = {}

local log = petbm.Debug:new("PetUtil")

local DEAL_DAMAGE_PLUS = 1
local DEAL_DAMAGE_MINUS = 2
local RECEIVE_DAMAGE_PLUS = 3
local RECEIVE_DAMAGE_MINUS = 4

local HUMANOID = petbm.PetConstants.HUMANOID
local DRAGON = petbm.PetConstants.DRAGON
local FLYING = petbm.PetConstants.FLYING
local UNDEAD = petbm.PetConstants.UNDEAD
local CRITTER = petbm.PetConstants.CRITTER
local MAGICAL = petbm.PetConstants.MAGICAL
local ELEMENTAL = petbm.PetConstants.ELEMENTAL
local BEAST = petbm.PetConstants.BEAST
local WATER = petbm.PetConstants.WATER
local MECHANICAL = petbm.PetConstants.MECHANICAL

local MODIFIER_TABLE = {
	{DRAGON, BEAST, UNDEAD, CRITTER}, -- HUMANOID
	{MAGICAL, UNDEAD, HUMANOID, FLYING}, -- DRAGON
	{WATER, DRAGON, MAGICAL, BEAST}, -- FLYING
	{HUMANOID, WATER, CRITTER, DRAGON}, -- UNDEAD
	{UNDEAD, HUMANOID, BEAST, nil}, -- CRITTER
	{FLYING, MECHANICAL, DRAGON, WATER}, -- MAGICAL
	{MECHANICAL, nil, WATER, MECHANICAL}, -- ELEMENTAL
	{CRITTER, FLYING, MECHANICAL, HUMANOID}, -- BEAST
	{ELEMENTAL, MAGICAL, FLYING, UNDEAD}, -- WATER
	{BEAST, ELEMENTAL, ELEMENTAL, MAGICAL} -- MECHANICAL
}

local function _Init(self)
	if (not self.rarityCache) then
		self.rarityCache = {}
	end
	if (not self.petTypes) then
		self.petTypes = {}
	end
	if (not self.petSources) then
		self.petSources = {}
	end
	if (not self.flags) then
		self.flags = {}
	end
end

local function _RestoreFilter(self)
	-- search text
	C_PetJournal.SetSearchFilter(self.searchText or "")

	-- pet types
	local petTypes = self.petTypes
	local numTypes = C_PetJournal.GetNumPetTypes()
	for i=1,numTypes do
		C_PetJournal.SetPetTypeFilter(i, petTypes[i])
	end
	
	-- pet sources
	local petSources = self.petSources
	local numSources = C_PetJournal.GetNumPetSources()
	for i=1,numSources do
		C_PetJournal.SetPetSourceFilter(i, petSources[i])
	end
	
	-- flags
	local flags = self.flags
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_COLLECTED, flags[LE_PET_JOURNAL_FLAG_COLLECTED])
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_FAVORITES, flags[LE_PET_JOURNAL_FLAG_FAVORITES])
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_NOT_COLLECTED, flags[LE_PET_JOURNAL_FLAG_NOT_COLLECTED])
end

local function _ClearFilter(self)
	-- search text
	self.searchText = petbm.PetBattleMaster.searchText
	
	-- pet types
	local petTypes = wipe(self.petTypes)
	local numTypes = C_PetJournal.GetNumPetTypes()
	for i=1,numTypes do
		petTypes[i] = not C_PetJournal.IsPetTypeFiltered(i)
	end
	
	-- pet sources
	local petSources = wipe(self.petSources)
	local numSources = C_PetJournal.GetNumPetSources()
	for i=1,numSources do
		petSources[i] = not C_PetJournal.IsPetSourceFiltered(i)
	end
	
	-- flags
	local flags = wipe(self.flags)
	flags[LE_PET_JOURNAL_FLAG_COLLECTED] = not C_PetJournal.IsFlagFiltered(LE_PET_JOURNAL_FLAG_COLLECTED)
	flags[LE_PET_JOURNAL_FLAG_FAVORITES] = not C_PetJournal.IsFlagFiltered(LE_PET_JOURNAL_FLAG_FAVORITES)
	flags[LE_PET_JOURNAL_FLAG_NOT_COLLECTED] = not C_PetJournal.IsFlagFiltered(LE_PET_JOURNAL_FLAG_NOT_COLLECTED)
	
	-- clear
	C_PetJournal.SetSearchFilter("")
	C_PetJournal.AddAllPetTypesFilter()
	C_PetJournal.AddAllPetSourcesFilter()
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_COLLECTED, true)
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_FAVORITES, false)
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_NOT_COLLECTED, false)
end

-- Returns dealDmgPlus, dealDmgMinus, receiveDmgPlus, receiveDmgMinus for the given pet type.
-- The modifiers will contain the target pet type. 
local function GetDamageModifiers(petType)
	return unpack(MODIFIER_TABLE[petType])
end

-- Read the current state of the pet journal. Should be called at startup and after 
-- having received PET_JOURNAL_LIST_UPDATE.
local function InitPetJournal()
	log:Debug("InitPetJournal")
	local self = petbm.PetUtil
	_Init(self)
	local rarityCache = wipe(self.rarityCache)
	
	_ClearFilter(self) 
--	log:Debug("Toggle") 
--	TogglePetJournal(2)
--	log:Debug("Toggle done")
	log:Debug("PetJournal [%s]", PetJournal)
	local numPets = C_PetJournal.GetNumPets(false)
	log:Debug("numPets [%s]", numPets)
	for i = 1,numPets do
		local petId, speciesId, owned = C_PetJournal.GetPetInfoByIndex(i, false)
		if (owned) then
			local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petId)
			local r = rarityCache[speciesId]
			if (r == nil or r < rarity) then
				if (rarity) then
					log:Debug("rarity [%s]", rarity)
				end
				rarityCache[speciesId] = rarity
			end
		end
	end
	_RestoreFilter(self)
--	log:Debug("toggle again")
--	TogglePetJournal(2)
--	log:Debug("InitPetJournal exit")
end

-- Returns the highest rarity for the given pet, or nil if not found.
local function GetPetRarityFromJournal(speciesId)
	local self = petbm.PetUtil
	_Init(self)
	local self = petbm.PetUtil 
	return self.rarityCache[speciesId]
end

petbm.PetUtil.GetDamageModifiers = GetDamageModifiers
petbm.PetUtil.InitPetJournal = InitPetJournal
petbm.PetUtil.GetPetRarityFromJournal = GetPetRarityFromJournal
